#include "Button.h"

#include "TextNode.h"
#include "GeometryNode.h"
#include "TransformNode.h"
#include "ThemeManager.h"
#include "PaintEvent.h"

#include "MouseEvent.h"
#include "FileLogger.h"

Button::Button(ButtonData::Component component)
    : Widget(),
      mLabel(),
      mClickFunc()
{
    mStyle.mComponent = component;
    mName = "Button";
}

Button::~Button()
{

}

std::unique_ptr<Button> Button::Create(ButtonData::Component component)
{
    return std::make_unique<Button>(component);
}

void Button::setOnClickFunction(clickFunc func)
{
    mClickFunc = func;
}

void Button::setLabel(const std::string& text)
{
    if (text != mLabel)
    {
        mLabel = text;
        mContentDirty = true;
    }
}

void Button::setState(ButtonData::State state)
{
    if (mStyle.mState != state)
    {
        mStyle.mState = state;
        mStateDirty = true;
    }
}

void Button::setEnabled(bool isEnabled)
{
    if (mEnabled != isEnabled)
    {
        mEnabled = isEnabled;
        if (mStyle.mState == ButtonData::State::Disabled)
        {
            setState(ButtonData::State::Enabled);
        }
    }
}

void Button::updateState()
{
    setBackground(mStyle.getContainerColor());

}

void Button::onMyMouseEvent(const MouseEvent* event)
{
    if (!mEnabled)
    {
        return;
    }

    if (event->getAction() == MouseEvent::Action::Pressed)
    {
        setState(ButtonData::State::Pressed);
    }
    else if (event->getAction() == MouseEvent::Action::Released)
    {
        resetState();
        if (mClickFunc)
        {
            mClickFunc(this);
        }
    }
    else if (event->getAction() == MouseEvent::Action::Enter)
    {
        resetState();
    }
    else if (event->getAction() == MouseEvent::Action::Leave)
    {
        resetState();
    }
}

void Button::resetState()
{
    if (mHasFocus)
    {
        setState(ButtonData::State::Focused);
    }
    else if (mContainsMouse)
    {
        setState(ButtonData::State::Hovered);
    }
    else if (mEnabled)
    {
        setState(ButtonData::State::Enabled);
    }
    else
    {
        setState(ButtonData::State::Disabled);
    }
}

bool Button::isDirty() const
{
    return Widget::isDirty() || mContentDirty;
}

void Button::doPaint(const PaintEvent* event)
{
    updateBackground(event);
    updateLabel(event);
}

void Button::updateLabel(const PaintEvent* event)
{
    unsigned fontOffset = unsigned(mLabel.size()) * 4;
    auto middle = DiscretePoint(mLocation.getX() + mSize.mWidth/2 - fontOffset, mLocation.getY() + mSize.mHeight/2 + 4);

    if (!mTextNode)
    {
        mTextNode = TextNode::Create(mLabel, middle);
        mTextNode->setName(mName + "_TextNode");
        mTextNode->setContent(mLabel);
        mTextNode->setWidth(mSize.mWidth);
        mTextNode->setHeight(mSize.mHeight);
        mRootNode->addChild(mTextNode.get());
    }

    if (mTransformDirty)
    {
        mTextNode->setLocation(middle);
        mTextNode->setWidth(mSize.mWidth);
        mTextNode->setHeight(mSize.mHeight);
    }

    if (mMaterialDirty)
    {
        mTextNode->setFillColor(event->getThemesManager()->getColor(mBackground));
    }

    if (mContentDirty)
    {
        mTextNode->setContent(mLabel);
        mContentDirty = false;
    }

    if (mVisibilityDirty)
    {
        mTextNode->setIsVisible(mVisible);
    }
}
